# frozen_string_literal: true

require 'bundler'
Bundler.require

class Collapser
  require_relative 'collapser/version'
  require_relative 'string'

  DEFAULT_REPLACER = 'коллапс'

  REPLACED_CHARS = [
    ('A'..'Z').to_a,
    ('А'..'Я').to_a,
    ('a'..'z').to_a,
    ('а'..'я').to_a
  ].flatten.freeze

  def initialize(text:, replacer: nil)
    @text_arr = text.split('')
    @replacer_arr = (replacer || DEFAULT_REPLACER).split('')
    @replacer_arr_idx = 0
  end

  attr_reader :text_arr, :replacer_arr

  def run
    text_arr.map { |char| convert_char(char: char) }.join
  end

  private

  attr_accessor :replacer_arr_idx

  def convert_char(char:)
    case char_type(char: char)
    when :other
      char
    when :downcase
      next_replace_char.downcase
    when :upcase
      next_replace_char.upcase
    end
  end

  def next_replace_char
    char = replacer_arr[replacer_arr_idx]
    increment_replacer_arr_idx!
    char
  end

  def char_type(char:)
    return :other unless REPLACED_CHARS.include?(char)
    return :downcase if char.downcase == char

    :upcase
  end

  def increment_replacer_arr_idx!
    self.replacer_arr_idx = replacer_arr_idx + 1
    self.replacer_arr_idx = 0 if replacer_arr_idx >= replacer_arr.size
    replacer_arr_idx
  end
end
